<?php
/* vim: set expandtab tabstop=4 shiftwidth=4: */
//
// +----------------------------------------------------------------------+
// | PHP Version 4                                                        |
// +----------------------------------------------------------------------+
// | Copyright (c) 1997-2002 The PHP Group                                |
// +----------------------------------------------------------------------+
// | This source file is subject to version 2.02 of the PHP license,      |
// | that is bundled with this package in the file LICENSE, and is        |
// | available at through the world-wide-web at                           |
// | http://www.php.net/license/3_0.txt.                                  |
// | If you did not receive a copy of the PHP license and are unable to   |
// | obtain it through the world-wide-web, please send a note to          |
// | license@php.net so we can mail you a copy immediately.               |
// +----------------------------------------------------------------------+
// | Authors: Harry Fuecks <hfuecks@phppatterns.com>                      |
// +----------------------------------------------------------------------+
//
// $Id: Calendar.php,v 1.8 2003/10/30 14:18:48 harryf Exp $
//
/**
 * @package Calendar
 * @version $Id: Calendar.php,v 1.8 2003/10/30 14:18:48 harryf Exp $
 */

/**
* Allows Calendar include path to be redefined
*/
if (!defined('CALENDAR_ROOT')) {
    define('CALENDAR_ROOT', 'Calendar'.DIRECTORY_SEPARATOR);
}

/**
 * Constant which defines the calculation engine to use
 */
if (!defined('CALENDAR_ENGINE')) {
    define('CALENDAR_ENGINE', 'UnixTS');
}

/**
 * Contains a factory method to return a Singleton instance of a class
 * implementing the Calendar_Engine_Interface.<br>
 * <b>Note:</b> this class must be modified to "register" alternative
 * Calendar_Engines. The engine used can be controlled with the constant
 * CALENDAR_ENGINE
 * @see Calendar_Engine_Interface
 * @package Calendar
 * @access protected
 */
class Calendar_Engine_Factory
{
    /**
     * Returns an instance of the engine
     * @return object instance of a calendar calculation engine
     * @access protected
     */
    function & getEngine()
    {
        static $engine = false;
        switch (CALENDAR_ENGINE) {
            case 'PearDate':
                $class = 'Calendar_Engine_PearDate';
                break;
            case 'UnixTS':
            default:
                $class = 'Calendar_Engine_UnixTS';
            break;
        }
        if (!$engine) {
            if ( !class_exists($class) ) {
                require_once CALENDAR_ROOT.'Engine/'.CALENDAR_ENGINE.'.php';
            }
            $engine = new $class;
        }
        return $engine;
    }
}

/**
 * Base class for Calendar API. This class should not be instantiated
 * directly.
 * @abstract
 * @package Calendar
 */
class Calendar
{
    /**
     * Instance of class implementing calendar engine interface
     * @var object
     * @access private
     */
    var $cE;

    /**
     * Instance of Calendar_Validator (lazy initialized when isValid() or
     * getValidor() is called
     * @var Calendar_Validator
     * @access private
     */
    var $validator;

    /**
     * Year for this calendar object e.g. 2003
     * @access private
     * @var int
     */
    var $year;

    /**
     * Month for this calendar object e.g. 9
     * @access private
     * @var int
     */
    var $month;

    /**
     * Day of month for this calendar object e.g. 23
     * @access private
     * @var int
     */
    var $day;

    /**
     * Hour of day for this calendar object e.g. 13
     * @access private
     * @var int
     */
    var $hour;

    /**
     * Minute of hour this calendar object e.g. 46
     * @access private
     * @var int
     */
    var $minute;

    /**
     * Second of minute this calendar object e.g. 34
     * @access private
     * @var int
     */
    var $second;

    /**
     * Marks this calendar object as selected (e.g. 'today')
     * @access private
     * @var boolean
     */
    var $selected = false;

    /**
     * Collection of child calendar objects created from subclasses
     * of Calendar. Type depends on the object which created them.
     * @access private
     * @var array
     */
    var $children = array();

    /**
     * Constructs the Calendar
     * @param int year
     * @param int month
     * @param int day
     * @param int hour
     * @param int minute
     * @param int second
     * @access protected
     */
    function Calendar($y = 2000, $m = 1, $d = 1, $h = 0, $i = 0, $s = 0)
    {
        static $cE = null;
        if (!isset($cE)) {
            $cE = & Calendar_Engine_Factory::getEngine();
        }
        $this->cE = & $cE;
        $this->year = (int)$y;
        $this->month = (int)$m;
        $this->day = (int)$d;
        $this->hour = (int)$h;
        $this->minute = (int)$i;
        $this->second = (int)$s;
    }

    /**
     * Defines the calendar by a Unix timestamp, replacing values
     * passed to the constructor
     * @param int Unix timestamp
     * @return void
     * @access public
     */
    function setTimestamp($ts)
    {
        $this->month  = $this->cE->stampToMonth($ts);
        $this->day    = $this->cE->stampToDay($ts);
        $this->hour   = $this->cE->stampToHour($ts);
        $this->minute = $this->cE->stampToMinute($ts);
        $this->second = $this->cE->stampToSecond($ts);
    }

    /**
     * Returns a timestamp from the current date / time values. Format of
     * timestamp depends on Calendar_Engine implementation being used
     * @return int timestamp
     * @access public
     */
    function getTimestamp()
    {
        return $this->cE->dateToStamp(
            $this->year, $this->month, $this->day,
            $this->hour, $this->minute, $this->second);
    }

    /**
     * Defines calendar object as selected (e.g. for today)
     * @param boolean state whether Calendar subclass
     * @return void
     * @access public
     */
    function setSelected($state = true)
    {
        $this->selected = $state;
    }

    /**
     * True if the calendar subclass object is selected (e.g. today)
     * @return boolean
     * @access public
     */
    function isSelected()
    {
        return $this->selected;
    }

    /**
     * Abstract method for building the children of a calendar object.
     * Implemented by Calendar subclasses
     * @param array containing Calendar objects to select (optional)
     * @return boolean
     * @access public
     * @abstract
     */
    function build($sDates = array())
    {
        require_once 'PEAR.php';
        PEAR::raiseError(
            'Calendar::build is abstract', null, PEAR_ERROR_TRIGGER,
            E_USER_NOTICE, 'Calendar::build()');
        return false;
    }

    /**
     * Abstract method for selected data objects called from build
     * @param array
     * @return boolean
     * @access public
     * @abstract
     */
    function setSelection($sDates)
    {
        require_once 'PEAR.php';
        PEAR::raiseError(
            'Calendar::setSelection is abstract', null, PEAR_ERROR_TRIGGER,
            E_USER_NOTICE, 'Calendar::setSelection()');
        return false;
    }

    /**
     * Iterator method for fetching child Calendar subclass objects
     * (e.g. a minute from an hour object). On reaching the end of
     * the collection, returns false and resets the collection for
     * further iteratations.
     * @return mixed either an object subclass of Calendar or false
     * @access public
     */
    function fetch()
    {
        $child = each($this->children);
        if ($child) {
            return $child['value'];
        } else {
            reset($this->children);
            return false;
        }
    }

    /**
     * Fetches all child from the current collection of children
     * @return array
     * @access public
     */
    function fetchAll()
    {
        return $this->children;
    }

    /**
     * Get the number Calendar subclass objects stored in the internal
     * collection.
     * @return int
     * @access public
     */
    function size()
    {
        return count($this->children);
    }

    /**
     * Determine whether this date is valid, with the bounds determined by
     * the Calendar_Engine. The call is passed on to
     * Calendar_Validator::isValid
     * @return boolean
     * @access public
     */
    function isValid()
    {
        $validator = & $this->getValidator();
        return $validator->isValid();
    }

    /**
     * Returns an instance of Calendar_Validator
     * @return Calendar_Validator
     * @access public
     */
    function & getValidator()
    {
        if (!isset($this->validator)) {
            require_once CALENDAR_ROOT.'Validator.php';
            $this->validator = & new Calendar_Validator($this);
        }
        return $this->validator;
    }

    /**
     * Returns a reference to the current Calendar_Engine being used. Useful
     * for Calendar_Table_Helper and Caledar_Validator
     * @return object implementing Calendar_Engine_Inteface
     * @access private
     */
    function & getEngine()
    {
        return $this->cE;
    }

    /**
     * Returns the value for the previous year
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 2002 or timestamp
     * @access public
     */
    function prevYear($asTs = false)
    {
        if (!$asTs) {
            return $this->year-1;
        } else {
            return $this->cE->dateToStamp($this->year-1, 1, 1, 0, 0, 0);
        }
    }

    /**
     * Returns the value for this year
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 2003 or timestamp
     * @access public
     */
    function thisYear($asTs = false)
    {
        if (!$asTs) {
            return $this->year;
        } else {
            return $this->cE->dateToStamp($this->year, 1, 1, 0, 0, 0);
        }
    }

    /**
     * Returns the value for next year
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 2004 or timestamp
     * @access public
     */
    function nextYear($asTs = false)
    {
        if (!$asTs) {
            return $this->year+1;
        } else {
            return $this->cE->dateToStamp($this->year+1, 1, 1, 0, 0, 0);
        }
    }

    /**
     * Returns the value for the previous month
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 4 or Unix timestamp
     * @access public
      */
    function prevMonth($asTs = false)
    {
        $ts = $this->cE->dateToStamp($this->year, $this->month-1, 1, 0, 0, 0);
        if (!$asTs) {
            return $this->cE->stampToMonth($ts);
        } else {
            return $ts;
        }
    }

    /**
     * Returns the value for this month
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 5 or timestamp
     * @access public
     */
    function thisMonth($asTs = false)
    {
        if (!$asTs) {
            return $this->month;
        } else {
            return $this->cE->dateToStamp($this->year, $this->month, 1, 0, 0, 0);
        }
    }

    /**
     * Returns the value for next month
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 6 or timestamp
     * @access public
     */
    function nextMonth($asTs = false)
    {
        $ts = $this->cE->dateToStamp($this->year, $this->month+1, 1, 0, 0, 0);
        if (!$asTs) {
            return $this->cE->stampToMonth($ts);
        } else {
            return $ts;
        }
    }

    /**
     * Returns the value for the previous day
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 10 or timestamp
     * @access public
     */
    function prevDay($asTs = false) {
        $ts = $this->cE->dateToStamp(
            $this->year, $this->month, $this->day-1, 0, 0, 0);
        if (!$asTs) {
            return $this->cE->stampToDay($ts);
        } else {
            return $ts;
        }
    }

    /**
     * Returns the value for this day
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 11 or timestamp
     * @access public
     */
    function thisDay($asTs=false)
    {
        if (!$asTs) {
            return $this->day;
        } else {
            return $this->cE->dateToStamp(
                $this->year, $this->month, $this->day, 0, 0, 0);
        }
    }

    /**
     * Returns the value for the next day
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 12 or timestamp
     * @access public
     */
    function nextDay($asTs=false)
    {
        $ts = $this->cE->dateToStamp(
            $this->year, $this->month, $this->day+1, 0, 0, 0);
        if (!$asTs) {
            return $this->cE->stampToDay($ts);
        } else {
            return $ts;
        }
    }

    /**
     * Returns the value for the previous hour
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 13 or timestamp
     * @access public
     */
    function prevHour($asTs=false)
    {
        $ts = $this->cE->dateToStamp(
            $this->year, $this->month, $this->day, $this->hour-1, 0, 0);
        if (!$asTs) {
            return $this->cE->stampToHour($ts);
        } else {
            return $ts;
        }
    }

    /**
     * Returns the value for this hour
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 14 or timestamp
     * @access public
     */
    function thisHour($asTs = false)
    {
        if (!$asTs) {
            return $this->hour;
        } else {
            return $this->cE->dateToStamp(
            $this->year, $this->month, $this->day, $this->hour, 0, 0);
        }
    }

    /**
     * Returns the value for the next hour
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 14 or timestamp
     * @access public
     */
    function nextHour($asTs = false)
    {
        $ts = $this->cE->dateToStamp(
            $this->year, $this->month, $this->day,$this->hour+1, 0, 0);
        if (!$asTs) {
            return $this->cE->stampToHour($ts);
        } else {
            return $ts;
        }
    }

    /**
     * Returns the value for the previous minute
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 23 or timestamp
     * @access public
     */
    function prevMinute($asTs = false)
    {
        $ts = $this->cE->dateToStamp(
            $this->year, $this->month, $this->day,
            $this->hour, $this->minute-1, 0);
        if (!$asTs) {
            return $this->cE->stampToMinute($ts);
        } else {
            return $ts;
        }
    }

    /**
     * Returns the value for this minute
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 24 or timestamp
     * @access public
     */
    function thisMinute($asTs = false)
    {
        if (!$asTs) {
            return $this->minute;
        } else {
            return $this->cE->dateToStamp(
                $this->year, $this->month, $this->day,
                $this->hour, $this->minute, 0);
        }
    }

    /**
     * Returns the value for the next minute
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 25 or timestamp
     * @access public
     */
    function nextMinute($asTs = false)
    {
        $ts = $this->cE->dateToStamp(
            $this->year, $this->month, $this->day,
            $this->hour, $this->minute+1, 0);
        if (!$asTs) {
            return $this->cE->stampToMinute($ts);
        } else {
            return $ts;
        }
    }

    /**
     * Returns the value for the previous second
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 43 or timestamp
     * @access public
     */
    function prevSecond($asTs = false)
    {
        $ts = $this->cE->dateToStamp(
            $this->year, $this->month, $this->day,
            $this->hour, $this->minute, $this->second-1);
        if (!$asTs) {
            return $this->cE->stampToSecond($ts);
        } else {
            return $ts;
        }
    }

    /**
     * Returns the value for this second
    * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 44 or timestamp
     * @access public
     */
    function thisSecond($asTs = false)
    {
        if (!$asTs) {
            return $this->second;
        } else {
            return $this->cE->dateToStamp(
                $this->year, $this->month, $this->day,
                $this->hour, $this->minute, $this->second);
        }
    }

    /**
     * Returns the value for the next second
     * @param boolean set to true to return a timestamp (optional)
     * @return int e.g. 45 or timestamp
     * @access public
     */
    function nextSecond($asTs = false)
    {
        $ts = $this->cE->dateToStamp(
                $this->year, $this->month, $this->day,
                $this->hour, $this->minute, $this->second+1);
        if (!$asTs) {
            return $this->cE->stampToSecond($ts);
        } else {
            return $ts;
        }
    }
}
?>